@@ -1,7 +1,30 @@
 ADPclustering=function(Data,ClusterNo=NULL,PlotIt=FALSE,...){
+  # INPUT
+  # Data[1:n,1:d]     Data set with n observations and d features
+  # ClusterNo         Number of clusters to search for
+  #
+  # OPTIONAL
+  # PlotIt            Boolean. Decision to plot or not
+  #
+  # OUTPUT
+  # Cls[1:n]          Clustering of data
+  # Object            Object of adpclust
+  #
+  # Author: MT, 04/2018
+  if (!requireNamespace('ADPclust')) {
+    message(
+      'Subordinate clustering package is missing. No computations are performed.
+            Please install the package which is defined in "Suggests".'
+    )
+    return(
+      list(
+        Cls = rep(1, nrow(Data)),
+        Object = "Subordinate clustering package is missing.
+                Please install the package which is defined in 'Suggests'."
+      )
+    )
+  }
   
-  #author: MT, 04/2018
-  requireNamespace('ADPclust')
   if(is.null(ClusterNo))
 	adp=ADPclust::adpclust(Data,...)
   else
